/*
** Programmer: Ciaran McHale (cjmchale@dsg.cs.tcd.ie)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <search.h>
#include <assert.h>
#include "globals.h"


typedef int (*cmp_fn_type)(const void *, const void *);

static int afm_cmp_char_names(const CharMetricInfo *, const CharMetricInfo *);



double
afm_string_width(my_FontInfo *f, const char *str, int font_size)
{
	long	res;

	res = 0;
	while (*str) {
		res += afm_char_width(f, *str);
	}
	return (res * font_size) / 1000.0;
}


int
afm_char_width(my_FontInfo *f, char code)
{
	unsigned char	ch;

	ch = (unsigned char)code;
	return f->cmi[ch].wx;
}


CharMetricInfo *
afm_find_char_info_of_code(my_FontInfo *f, char code)
{
	unsigned char	ch;

	ch = (unsigned char)code;
	return &(f->cmi[ch]);
}


CharMetricInfo *
afm_find_char_info_of_glyph(my_FontInfo *f, const char *name)
{
	CharMetricInfo key;

	key.name = (char *)name;
	return (CharMetricInfo *) lfind(&key,
					f->f->cmi, &(f->f->numOfChars),
					sizeof(CharMetricInfo),
					(cmp_fn_type)afm_cmp_char_names);
}


static int
afm_cmp_char_names(const CharMetricInfo *c1, const CharMetricInfo *c2)
{
	return strcmp(c1->name, c2->name);
}


double
afm_capHeight(my_FontInfo *f, int font_size)
{
	return f->f->gfi->capHeight * font_size / 1000.0;
}


void
destroy_my_FontInfo(my_FontInfo *f, BOOL delete_f)
{
	if (f->f != NULL) {
		if (f->f->gfi != NULL) {
			free(f->f->gfi->afmVersion);
			f->f->gfi->afmVersion = NULL;
			free(f->f->gfi->fontName);
			f->f->gfi->fontName = NULL;
			free(f->f->gfi->fullName);
			f->f->gfi->fullName = NULL;
			free(f->f->gfi->familyName);
			f->f->gfi->familyName = NULL;
			free(f->f->gfi->weight);
			f->f->gfi->weight = NULL;
			free(f->f->gfi->version);
			f->f->gfi->version = NULL;
			free(f->f->gfi->notice);
			f->f->gfi->notice = NULL;
			free(f->f->gfi->encodingScheme);
			f->f->gfi->encodingScheme = NULL;
			free(f->f->gfi);
			f->f->gfi = NULL;
		}

		if (f->f->cwi != NULL) {
			free(f->f->cwi);
			f->f->cwi = NULL;
		}

		if (f->f->cmi != NULL)
		{ 
			int i = 0;
			CharMetricInfo *temp = f->f->cmi;
			Ligature *node = temp->ligs;

			for (i = 0; i < f->f->numOfChars; ++i)
			{
			for (node= temp->ligs; node != NULL; node= node->next) {
				free(node->succ);
				node->succ = NULL;
				free(node->lig);
				node->lig = NULL;
			}
			free(temp->name);
			temp->name = NULL;
			temp++;
		}
		free(f->f->cmi);
		f->f->cmi = NULL;
	}

	if (f->f->tkd != NULL) {
		free(f->f->tkd);
		f->f->tkd = NULL;
	}

	if (f->f->pkd != NULL) {
		free(f->f->pkd->name1);
		f->f->pkd->name1 = NULL;
		free(f->f->pkd->name2);
		f->f->pkd->name2 = NULL;
		free(f->f->pkd);
		f->f->pkd = NULL;
	}

	if (f->f->ccd != NULL) {
		int i = 0, j = 0;
		CompCharData *ccd = f->f->ccd;

		for (i = 0; i < f->f->numOfComps; ++i) {
				for (j = 0; j < ccd[i].numOfPieces; ++j) {
					free(ccd[i].pieces[j].pccName); 
					ccd[i].pieces[j].pccName = NULL;
				}
				free(ccd[i].ccName);
				ccd[i].ccName = NULL;
			}
			free(f->f->ccd);
			f->f->ccd = NULL;
		}
		free(f->f);
	}

	if (delete_f) {
		free(f);
	}
}
