/*
** Programmer: Ciaran McHale (cjmchale@dsg.cs.tcd.ie)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"




static FILE *
open_file_in_path(const char *file_name, const char *path)
{
	FILE	*res;
	char	*file;
	char	*copy, *p;
	char	*dir;


	/*
	** Create a copy of the path that we can overwrite as we please
	*/
	copy = (char *)my_malloc(strlen(path) + 1);
	strcpy(copy, path);

	/*
	** Create enough scratch space for the name of a file which
	** is (hopefully) located somewhere in the path
	*/
	file = my_malloc(strlen(path) + strlen(file_name) + 2);

	/*
	** Cycle through the path, looking for the file
	*/
	p = copy;
	while ((dir = strtok(p, PATH_SEP_STR)) != NULL) {
		p = NULL;
		if (strlen(dir) && dir[strlen(dir)-1] != DIR_SEP_CHAR) {
			sprintf(file, "%s%c%s", dir, DIR_SEP_CHAR, file_name);
		} else {
			sprintf(file, "%s%s", dir, file_name);
		}
		res = fopen(file, "r");
		if (res) {
			free(file);
			free(copy);
			return res;
		}
	}
	free(file);
	free(copy);
	return NULL;
}


FILE *
open_font_file(const char *file_name, const char *ext)
{
	char	name[100];
	char	*path;

	path = getenv("PAL_FONT_PATH");
	if (!path) {
		path = DEFAULT_FONT_PATH;
	}
	sprintf(name, "%s.%s", file_name, ext);
	return open_file_in_path(name, path);
}


FILE *
open_lib_file(const char *file_name, const char *description)
{
	char	*path;
	FILE	*res;

	path = getenv("PAL_LIB_PATH");
	if (!path) {
		path = DEFAULT_LIB_PATH;
	}
	res = open_file_in_path(file_name, path);
	if (!res) {
		fprintf(stderr,
			"Could not open the %s `%s'.\n%s\n",
			description,
			file_name,
			"Check setting of PAL_LIB_PATH environment variable"
			);
		exit(1);
	}
	return res;
}
