/*
** Programmer: Ciaran McHale (cjmchale@dsg.cs.tcd.ie)
*/

#include <stdio.h>
#include <string.h>
#include "globals.h"


typedef struct {
	char	*input_code;
	char	*glyph;
	int	code;
} AccentInfo;

static AccentInfo accents[] =
{
/* 030x */
	{ "`A",		"Agrave",		0300 },
	{ "'A",		"Aacute",		0301 },
	{ "^A",		"Acircumflex",		0302 },
	{ "~A",		"Atilde",		0303 },
	{ "\"A",	"Adieresis",		0304 },
	{ "oA",		"Aring",		0305 },

	{ ",C",		"Ccedilla",		0307 },

/* 031x */
	{ "`E",		"Egrave",		0310 },
	{ "'E",		"Eacute",		0311 },
	{ "^E",		"Ecircumflex",		0312 },
	{ "\"E",	"Edieresis",		0313 },

	{ "`I",		"Igrave",		0314 },
	{ "'I",		"Iacute",		0315 },
	{ "^I",		"Icircumflex",		0316 },
	{ "\"I",	"Idieresis",		0317 },

/* 032x */

	{ "~N",		"Ntilde",		0321 },

	{ "`O",		"Ograve",		0322 },
	{ "'O",		"Oacute",		0323 },
	{ "^O",		"Ocircumflex",		0324 },
	{ "~O",		"Otilde",		0325 },
	{ "\"O",	"Odieresis",		0326 },

/* 033x */

	{ "`U",		"Ugrave",		0331 },
	{ "'U",		"Uacute",		0332 },
	{ "^U",		"Ucircumflex",		0333 },
	{ "\"U",	"Udieresis",		0334 },

	{ "'Y",		"Yacute",		0335 },

/* 034x */
	{ "`a",		"agrave",		0340 },
	{ "'a",		"aacute",		0341 },
	{ "^a",		"acircumflex",		0342 },
	{ "~a",		"atilde",		0343 },
	{ "\"a",	"adieresis",		0344 },
	{ "oa",		"aring",		0345 },

	{ ",c",		"ccedilla",		0347 },

/* 035x */

	{ "`e",		"egrave",		0350 },
	{ "'e",		"eacute",		0351 },
	{ "^e",		"ecircumflex",		0352 },
	{ "\"e",	"edieresis",		0353 },

	{ "`i",		"igrave",		0354 },
	{ "'i",		"iacute",		0355 },
	{ "^i",		"icircumflex",		0356 },
	{ "\"i",	"idieresis",		0357 },

/* 036x */

	{ "~n",		"ntilde",		0361 },

	{ "`o",		"ograve",		0362 },
	{ "'o",		"oacute",		0363 },
	{ "^o",		"ocircumflex",		0364 },
	{ "~o",		"otilde",		0365 },
	{ "\"o",	"odieresis",		0366 },

/* 037x */

	{ "`u",		"ugrave",		0371 },
	{ "'u",		"uacute",		0372 },
	{ "^u",		"ucircumflex",		0373 },
	{ "\"u",	"udieresis",		0374 },

	{ "'y",		"yacute",		0375 },
};

static const int	num_accents = sizeof(accents) / sizeof(accents[0]);


char
find_accented_char(char c0, char c1)
{
	int	i;

	for (i = 0; i < num_accents; i++) {
		AccentInfo	*a = &accents[i];

		if (a->input_code[0] == c0 && a->input_code[1] == c1) {
			/*
			** Found it in accents[].
			** But, does the font actually have it?
			*/
			if (afm_find_char_info_of_glyph(&g.f, a->glyph)) {
				/* Yes, the font has it */
				return a->code;
			}
		}
	}
	/* not found */
	return 0;
}


void
check_and_reencode_font(my_FontInfo *f)
{
	CharMetricInfo	*unknown, *c;
	int		i;

#if	0
	/*
	** First check that the font has an encoding suitable for our purposes
	*/
	if (   strcmp(f->f->gfi->encodingScheme,  ADOBE_ENCODING)
	    && strcmp(f->f->gfi->encodingScheme, LATIN1_ENCODING)
	    && !g.quiet_mode)
	{
		fprintf(stderr, "%s: %s `%s' is `%s'.\n%s `%s' or `%s'.\n%s\n",
			g.prog_name,
			"Warning: the encoding of the font",
			f->f->gfi->fontName,
			f->f->gfi->encodingScheme,
			"This program works best with",
			ADOBE_ENCODING,
			LATIN1_ENCODING,
			"Proceeding at your own risk..."
			);
	}
#endif

	unknown = afm_find_char_info_of_glyph(f, ".notdef");
	if (!unknown) {
		unknown = afm_find_char_info_of_glyph(f, "space");
	}
	for (i = 0; i < 256; i++) {
		f->cmi[i] = *unknown;
	}
	for (i = 0; i < f->f->numOfChars; i++) {
		int	code = f->f->cmi[i].code;

		if (code != -1) {
			f->cmi[code] = f->f->cmi[i];
		}
	}
	if (strcmp(f->f->gfi->encodingScheme, LATIN1_ENCODING)) {
		/*
		** Make the accents accessible
		*/
		for (i = 0; i < num_accents; i++) {
			c = afm_find_char_info_of_glyph(f, accents[i].glyph);
			if (c) {
				f->cmi[ accents[i].code ] = *c;
			}
		}
	} else {
		/*
		** The LATIN1_ENCODING already has these glyphs in place
		** so no reencoding is necessary.
		*/
	}
}
