/*
** Programmer: Ciaran McHale (cjmchale@dsg.cs.tcd.ie)
*/

#include <stdio.h>
#include "globals.h"


void
generate_setup_section(PS_generator psg, FILE *font_file, BOOL is_pfa_file)
{
	char	str[256];
	int	i;

	/*
	** Generate %%Begin(End)Setup section of PostScript file
	*/
	ps_gen_DSC_comment(psg, "%%BeginSetup");
	ps_gen_command(psg, "pal_dict begin");
	if (font_file) {
		/*
		** Include the font in the PS file
		*/
		sprintf(str, "%s%s",
			"%%BeginResource: font ", g.f.f->gfi->fontName);
		ps_gen_DSC_comment(psg, str);
		if (is_pfa_file) {
			ps_gen_append_ascii_file(psg, font_file);
		} else {
			ps_gen_append_pfb_file(psg, font_file);
		}
		ps_gen_DSC_comment(psg, "%%EndResource");
	} else {
		/*
		** Generate a DSC comment to specify that the font ought to
		** be included here
		*/
		sprintf(str, "%s %s",
			"%%IncludeResource: font", g.f.f->gfi->fontName);
		ps_gen_DSC_comment(psg, str);
	}
	ps_gen_newline(psg);
	ps_gen_comment(psg,"%Reencode the font for access to accented chars");
	sprintf(str, "/%s /font accented_chars ReEncodeSmall",
		g.f.f->gfi->fontName);
	ps_gen_command(psg, str);
	ps_gen_newline(psg);
	ps_gen_newline(psg);
	ps_gen_comment(psg,"%To avoid repeated invocations of `scalefont' we");
	ps_gen_comment(psg,"%cache the font at some (hopefully) popular sizes");
	sprintf(str, "/font_1pt /font findfont def");
	ps_gen_command(psg, str);
	ps_gen_newline(psg);
	for (i = 0; i < 5 && g.config.max_font_size - i > 0; i++) {
		sprintf(str, "/fc%d font_1pt %d scalefont def",
			i, g.config.max_font_size - i);
		ps_gen_command(psg, str);
		ps_gen_newline(psg);
	}
	if (g.draw_box) {
		sprintf(str, "%f setlinewidth", g.box_thickness);
		ps_gen_command(psg, str);
	}
	ps_gen_DSC_comment(psg, "%%EndSetup");
}
