/*
** Programmer: Ciaran McHale (cjmchale@dsg.cs.tcd.ie)
*/

#include "config.h"

#include "parseAFM.h"
/*
** "parseAFM.h" has #defines for BOOL, TRUE and FALSE so no need to
** define them here.
*/

typedef struct {
	FontInfo	*f;
	CharMetricInfo	cmi[256];	/* for quicker access to char info */
} my_FontInfo;

#include "ps_gen.h"
#include "psbbox.h"

#ifndef __hpux
extern	int	getopt(int argc, char *argv[], char *optstring);
extern	char	*optarg;
extern	int	optind, opterr;
#endif

#define MAX_LEN		1024	/* max string length */

#define  ADOBE_ENCODING	"AdobeStandardEncoding"
#define LATIN1_ENCODING	"ISOLatin1Encoding"


/*
** Functions in "my_malloc.c"
*/
extern	void	*my_malloc(int);
extern	void	*my_realloc(void *ptr, int size);
extern	char	*str_malloc(char *str);


/*
** Functions in "files_stuff.c"
*/
extern	FILE	*open_lib_file(const char *file_name, const char *description);
extern	FILE	*open_font_file(const char *file_name, const char *ext);


/*
** Functions in "fonts_stuff.c"
*/
extern	void	check_and_reencode_font(my_FontInfo *f);
extern	char	find_accented_char(char c0, char c1);


/*
** Functions in "configFile.c"
*/
extern	void	parse_config_file(FILE *cfile);
extern	void	print_config_file_values(void);
extern	void	check_config_file_values(void);


/*
** Functions in "gen_setup.c"
*/
extern	void	generate_setup_section(PS_generator psg, FILE *font_file,
					BOOL is_pfa_file);


/*
** Functions in "afm_routines.c"
*/
void		destroy_my_FontInfo(my_FontInfo *f, BOOL delete_f);
CharMetricInfo	*afm_find_char_info_of_glyph(my_FontInfo *f, const char *name);
CharMetricInfo	*afm_find_char_info_of_code(my_FontInfo *f, char code);
int		afm_char_width(my_FontInfo *f, char code);
double		afm_string_width(my_FontInfo *f,const char *str, int font_size);
double		afm_capHeight(my_FontInfo *f, int font_size);


typedef struct {
	/*
	** Variables to store the actual text of the label
	*/
	char	**lines;
	int	num_lines;
	int	max_lines;

	/*
	** Variables to hold information needed to reproduce the
	** label on the output page
	*/
	int	font_size;
	double	width;	/* width and height are that of a rectangle */
	double	height; /* just big enought to enclose the text */
	BOOL	okay;
} LabelInfo;


/*
** Functions in "processInput.c"
*/
extern	void	process_input_normally(PS_generator psg, FILE *in, int *pages);
extern	void	process_input_page_per_label(PS_generator psg, FILE *in,
					     int *pages);


/*
** Functions in "processLabel.c"
*/
extern	void	process_label(LabelInfo *label);
extern	void	print_label(PS_generator psg, LabelInfo *label, int label_num);
extern	double	font_spacing(int font_size);
extern	double	label_center_height(LabelInfo *label, int font_size);
extern	void	reg_font_size(int font_size);




struct ConfigFileStuff {
	double	page_width;
	double	page_height;
	double	left_margin;
	double	top_margin;
	double	label_width;
	double	label_height;
	double	h_inside_margin;
	double	v_inside_margin;
	double	h_gap;
	double	v_gap;
	int	num_rows;
	int	num_cols;
	int	max_font_size;
	char	*font_file;
};


/*
** Put all global variables into a single structure so we
** can keep an eye on them.
*/
struct GlobalsType {
	char			*prog_name;	/* argv[0] */
	my_FontInfo		f;
	BOOL			draw_box;
	double			box_thickness;
	BOOL			include_font;
	BOOL			page_per_label;
	BOOL			quiet_mode;
	PSBBox			doc_psbbox;
	PSBBox			page_psbbox;
	struct ConfigFileStuff	config;
};

extern	struct GlobalsType	g;
