/*
** Programmer: Ciaran McHale (cjmchale@dsg.cs.tcd.ie)
**
** A data structure and associated routines to make it easier to keep
** track of bounding box information.
*/

#include "psbbox.h"
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>

extern	void	*my_malloc(int);


struct _PSBBox {
	double	x1, y1,
		x2, y2;
};


PSBBox
make_empty_psbbox(void)
{
	PSBBox	b;

	b = (PSBBox)my_malloc(sizeof(struct _PSBBox));
	b->x1 = 0.0;	b->y1 = 0.0;
	b->x2 = 0.0;	b->y2 = 0.0;
	return b;
}


PSBBox
make_psbbox(double x1, double y1, double x2, double y2)
{
	PSBBox	b;

	b = (PSBBox)my_malloc(sizeof(struct _PSBBox));
	b->x1 = x1;	b->y1 = y1;
	b->x2 = x2;	b->y2 = y2;
	return b;
}


void
destroy_psbbox(PSBBox b)
{
	free(b);
}


void
psbbox_union(const PSBBox b1, const PSBBox b2, PSBBox result)
{
	if (psbbox_is_empty(b1)) {
		psbbox_assign(result, b2);
	} else if (psbbox_is_empty(b2)) {
		psbbox_assign(result, b1);
	} else {
#define DMIN(a, b)	( (a) < (b) ? a : b )
#define DMAX(a, b)	( (a) > (b) ? a : b )
		result->x1 = DMIN(b1->x1, b2->x1);
		result->y1 = DMIN(b1->y1, b2->y1);
		result->x2 = DMAX(b1->x2, b2->x2);
		result->y2 = DMAX(b1->y2, b2->y2);
#undef DMIN
	}
}


BOOL
psbbox_is_empty(const PSBBox b)
{
	return (b->x1 == b->y1 && b->x1 == b->x2 && b->x1 == b->y2);
}


void
psbbox_get_corners(const PSBBox b, double *x1, double *y1, double *x2, double *y2)
{
	*x1 = b->x1;	*y1 = b->y1;
	*x2 = b->x2;	*y2 = b->y2;
}


void
psbbox_assign(PSBBox result, const PSBBox b)
{
	result->x1 = b->x1;	result->y1 = b->y1;
	result->x2 = b->x2;	result->y2 = b->y2;
}
